@@ -14,6 +14,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+DEFAULT_VELOCITY = 100
+DEFAULT_VOLUME = 100
+
 PPPP = 30
 PPP  = 35
 PP   = 40
