@@ -22,10 +22,10 @@ import const
 from rat import Rat
 
 class MusicDisplayer(gtk.ScrolledWindow):
-    def __init__(self, callback):
+    def __init__(self):
         gtk.ScrolledWindow.__init__(self)
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
-        self.m_callback = callback
+        self.m_callback = None
         self.m_engravers = [[],]
         self.m_fontsize = 20
         self.m_clickables = []
@@ -83,10 +83,11 @@ class MusicDisplayer(gtk.ScrolledWindow):
         self.m_clickables.append({'x':x, 'y':y, 'w':w, 'h':h,
                                   'midi_int': midi_int})
     def on_button_press_event(self, arg1, event):
-        for r in self.m_clickables:
-            if r['x'] < event.x < r['x'] + r['w'] \
-               and r['y'] < event.y < r['y'] + r['h']:
-                self.m_callback(r['midi_int'])
+        if self.m_callback:
+            for r in self.m_clickables:
+                if r['x'] < event.x < r['x'] + r['w'] \
+                   and r['y'] < event.y < r['y'] + r['h']:
+                    self.m_callback(r['midi_int'])
     def on_expose_event(self, darea, event):
         self.white_gc = self.g_d.get_style().white_gc
         self.black_gc = self.g_d.get_style().black_gc
@@ -123,7 +124,7 @@ class MusicDisplayer(gtk.ScrolledWindow):
 
 class ChordEditor(MusicDisplayer):
     def __init__(self):
-        MusicDisplayer.__init__(self, None)
+        MusicDisplayer.__init__(self)
         self._yp = None
         self.g_d.connect("button_release_event", self.on_button_release_event)
         self.g_d.connect("event", self.on_event)
