@@ -16,7 +16,7 @@
 
 import os
 
-class SoundInitException:
+class SoundInitException(Exception):
     pass
 class SyscallException(SoundInitException):
     def __init__(self, msg, n):
