@@ -29,6 +29,7 @@ import traceback
 import cfg
 import lessonfile
 import osutils
+import const
 
 class QstatusDefs:
     QSTATUS_NO = 0
@@ -224,24 +225,6 @@ class MelodicIntervalTeacher(Teacher):
 
 
 class RhythmAddOnClass:
-    RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
-               "c16 c16 c8", "c16 c8 c16", "c8. c16", "c16 c8.",
-               "r4", "r8c8", "r8 c16 c16", "r16 c16 c8", "r16c8c16",
-               "r16 c16 c16 c16", "r8 r16 c16", "r16 c8.",
-               "c12 c12 c12", "r12 c12 c12",
-               "c12 r12 c12", "c12 c12 r12", "r12 r12 c12", "r12 c12 r12",
-               "c4.", "c4 c8", # 22, 23
-               "c8 c4", "c8 c8 c8", # 24, 25
-               "c4 c16 c16", # 26
-               "c16 c16 c4", # 27
-               "c8 c8 c16 c16", #28
-               "c8 c16 c16 c8", #29
-               "c16 c16 c8 c8", #30
-               "c8 c16 c16 c16 c16", #31
-               "c16 c16 c8 c16 c16", #32
-               "c16 c16 c16 c16 c8", #33
-               "c16 c16 c16 c16 c16 c16", #34
-               )
     def new_question(self):
         """returns:
                self.ERR_PICKY : if the question is not yet solved and the
@@ -263,7 +246,7 @@ class RhythmAddOnClass:
         norest_v = []
         v = []
         for x in self.m_P.header.rhythm_elements:
-            if not (self.RHYTHMS[x][0] == "r"
+            if not (const.RHYTHMS[x][0] == "r"
                     and self.get_bool("not_start_with_rest")):
                 norest_v.append(x)
             v.append(x)
@@ -288,7 +271,7 @@ class RhythmAddOnClass:
             else:
                 count_in_notelen = "4"
             s = "d%s " % count_in_notelen * self.get_int("count_in")
-        s += " ".join([self.RHYTHMS[k] for k in self.m_question])
+        s += " ".join([const.RHYTHMS[k] for k in self.m_question])
         return s
     def get_music_string(self):
         """
@@ -578,7 +561,7 @@ class RhythmAddOnGuiClass(object):
         table.show_all()
     def pngcheckbutton(self, i):
         btn = gtk.CheckButton()
-        btn.add(gu.create_rhythm_image(RhythmAddOnClass.RHYTHMS[i]))
+        btn.add(gu.create_rhythm_image(const.RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.select_element_cb, i)
         return btn
