@@ -97,9 +97,8 @@ def split(key):
 
 def parse_file_into_dict(dictionary, filename):
     f = codecs.open(filename, "rU", "utf-8")
-    line = f.readline()
     section = None
-    while line:
+    for lineno, line in enumerate(f):
         section_m = section_re.match(line)
         value_m = value_re.match(line)
         comment_m = comment_re.match(line)
@@ -115,9 +114,7 @@ def parse_file_into_dict(dictionary, filename):
         elif not line.strip():
             pass
         else:
-            print filename
-            raise Exception("Shit happended line %s" % line)
-        line = f.readline()
+            raise CfgParseException(filename, lineno)
     f.close()
     return dictionary
 
@@ -306,6 +303,14 @@ def reread_data():
     if _user_filename and os.path.isfile(_user_filename):
         data = parse_file_into_dict(data, _user_filename)
 
+class CfgParseException(Exception):
+    def __init__(self, filename, lineno):
+        Exception.__init__(self)
+        self.m_filename = filename
+        self.m_lineno = lineno
+    def __str__(self):
+        return "CfgParseException: Failed to parse line %(lineno)i of the file '%(filename)s'" % {'lineno': self.m_lineno, 'filename': self.m_filename}
+
 def initialise(app_defaults_filename, system_filename, user_filename):
     """
     app_defaults_filename: file must exist!
@@ -323,8 +328,13 @@ def initialise(app_defaults_filename, system_filename, user_filename):
     if app_defaults_filename:
         try:
             data = parse_file_into_dict({}, app_defaults_filename)
-        except:
-            raise Exception("Solfege install is completely broken.\nCannot find required file %s" % app_defaults_filename)
+        except CfgParseException, e:
+            print "-" * 60
+            print "The program failed to parse default.config. This means that"
+            print "you have modified the file, because it was ok then Solfege"
+            print "was released. Fix the file yourself, or reinstall Solfege."
+            print "-" * 60
+            raise
     else:
         data = {}
     if system_filename and os.path.isfile(system_filename):
