@@ -148,7 +148,7 @@ class MyTextView(gtk.TextView):
                     # data variable 'html-percentage-width' set.
                     if width.endswith('%'):
                         width = int(float(width[:-1]))
-                        widget.set_size_request(parent_w * width / 100, -1)
+                        widget.set_size_request((parent_w - self.m_margin * 2) * width / 100, -1)
                     else:
                         width = int(float(width))
                         widget.set_size_request(width, -1)
