@@ -15,7 +15,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 import errno
 import tempfile
 import textwrap
@@ -27,6 +26,7 @@ import time
 import locale
 import filesystem
 import utils
+import subprocess
 
 import re, os, sys
 import random
@@ -208,6 +208,7 @@ lessonfile_functions = {
         # play_wav should probably be removed. Replaced by wavfile
         'play_wav': (False, lambda f: Wavfile(f)),
         'music': (False, lambda m: Music(m)),
+        'music3': (False, lambda m: Music3(m)),
         'chord': (False, lambda m: Chord(m)),
         'satb': (False, lambda m: Satb(m)),
         'voice': (False, lambda m: Voice(m)),
@@ -351,14 +352,21 @@ class MpdParsable(MusicBaseClass):
     """
     def play(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
-            lessonfile_ref.get_tempo(), instrument[0], instrument[1])
+        instrument = lessonfile_ref.get_instruments(question, 1)
+        if len(instrument) == 2:
+            mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
+                lessonfile_ref.get_tempo(),
+                instrument[0],
+                instrument[1])
+        else:
+            mpd.play_music3(self.get_mpd_music_string(lessonfile_ref),
+                lessonfile_ref.get_tempo(),
+                instrument)
         if _test_mode:
             return self.get_mpd_music_string(lessonfile_ref)
     def play_slowly(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
+        instrument = lessonfile_ref.get_instrument(question)
         tempo = lessonfile_ref.get_tempo()
         tempo = (tempo[0]/2, tempo[1])
         mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
@@ -391,8 +399,6 @@ class ChordCommon(MpdTransposable):
     pass
 
 class Chord(ChordCommon):
-    def __init__(self, musicdata):
-        ChordCommon.__init__(self, musicdata)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\score{ "\
                r" \transpose c' %s{ <%s> }"\
@@ -446,61 +452,53 @@ class Chord(ChordCommon):
                 (question.tempo[0] /2, question.tempo[1]))
     def __play(self, lessonfile_ref, question, tempo):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        assert len(instrument) in (2, 6)
-        if len(instrument) == 2:
-            mpd.play_music(self.get_mpd_music_string(lessonfile_ref),
-                tempo, instrument[0], instrument[1])
-        else:
-            assert len(instrument) == 6
-            t1 = soundcard.Track()
-            t2 = soundcard.Track()
-            t3 = soundcard.Track()
-            t1.set_bpm(tempo[0])
-            nlist = self.get_music_as_notename_list(lessonfile_ref)
-            t1.set_patch(instrument[0])
-            t2.set_patch(instrument[2])
-            t3.set_patch(instrument[4])
-            # start notes
-            t1.note(4, mpd.notename_to_int(nlist[0]), instrument[1])
-            for notename in nlist[1:-1]:
-                t2.start_note(mpd.notename_to_int(notename), instrument[3])
-            t2.notelen_time(4)
-            for notename in nlist[1:-1]:
-                t2.stop_note(mpd.notename_to_int(notename), instrument[3])
-            t3.note(4, mpd.notename_to_int(nlist[-1]), instrument[5])
-            soundcard.synth.play_track(t1, t2, t3)
+        instrument = lessonfile_ref.get_instruments(question, 3)
+        t1, t2, t3 = utils.new_3_tracks()
+        t1.set_bpm(tempo[0])
+        nlist = self.get_music_as_notename_list(lessonfile_ref)
+        t1.set_patch(instrument[0])
+        t1.set_volume(instrument[1])
+        t2.set_patch(instrument[2])
+        t2.set_volume(instrument[3])
+        t3.set_patch(instrument[4])
+        t3.set_volume(instrument[5])
+        # start notes
+        t1.note(4, mpd.notename_to_int(nlist[0]))
+        for notename in nlist[1:-1]:
+            t2.start_note(mpd.notename_to_int(notename))
+        t2.notelen_time(4)
+        for notename in nlist[1:-1]:
+            t2.stop_note(mpd.notename_to_int(notename))
+        t3.note(4, mpd.notename_to_int(nlist[-1]))
+        soundcard.synth.play_track(t1, t2, t3)
     def play_arpeggio(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         # We have a problem here because the music need to know
         # things from the question it belongs to.
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        assert len(instrument) in (2, 6)
-        if len(instrument) == 2:
-            m = self.get_music_as_notename_string(lessonfile_ref)
-            mpd.play_music(r"\staff{%s}" % m, cfg.get_int('config/arpeggio_bpm'),
-                       instrument[0], instrument[1])
-        else:
-            assert len(instrument) == 6
-            t1 = soundcard.Track()
-            t2 = soundcard.Track()
-            t3 = soundcard.Track()
-            t1.set_bpm(cfg.get_int('config/arpeggio_bpm'))
-            nlist = self.get_music_as_notename_list(lessonfile_ref)
-            # set patches
-            t1.set_patch(instrument[0])
-            t2.set_patch(instrument[2])
-            t3.set_patch(instrument[4])
-            # start notes
-            t1.note(4, mpd.notename_to_int(nlist[0]), instrument[1])
-            t2.notelen_time(4)
+        instrument = lessonfile_ref.get_instruments(question, 3)
+        assert len(instrument) == 6
+        t1, t2, t3 = utils.new_3_tracks()
+        t1.set_bpm(cfg.get_int('config/arpeggio_bpm'))
+        nlist = self.get_music_as_notename_list(lessonfile_ref)
+        # set patches and volumes
+        t1.set_patch(instrument[0])
+        t1.set_volume(instrument[1])
+        t2.set_patch(instrument[2])
+        t2.set_volume(instrument[3])
+        t3.set_patch(instrument[4])
+        t3.set_volume(instrument[5])
+        # start notes
+        t1.note(4, mpd.notename_to_int(nlist[0]))
+        t2.notelen_time(4)
+        t3.notelen_time(4)
+        for notename in nlist[1:-1]:
+            t2.note(4, mpd.notename_to_int(notename))
             t3.notelen_time(4)
-            for notename in nlist[1:-1]:
-                t2.note(4, mpd.notename_to_int(notename), instrument[3])
-                t3.notelen_time(4)
-            t3.note(4, mpd.notename_to_int(nlist[-1]), instrument[5])
-            soundcard.synth.play_track(t1, t2, t3)
-
+        t3.note(4, mpd.notename_to_int(nlist[-1]))
+        soundcard.synth.play_track(t1, t2, t3)
+    def get_err_context(self, exception, lessonfile_ref):
+        return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
+            " " * exception.m_linepos1 + "^" * (exception.m_linepos2 - exception.m_linepos1)))
 
 class VoiceCommon(MpdTransposable):
     def get_first_pitch(self):
@@ -508,8 +506,6 @@ class VoiceCommon(MpdTransposable):
         return self.m_musicdata[a:b]
 
 class Voice(VoiceCommon):
-    def __init__(self, musicdata):
-        VoiceCommon.__init__(self, musicdata)
     def get_lilypond_code(self, lessonfile_ref):
         return r"\transpose c' %s{ %s }" % (
             lessonfile_ref.m_transpose.get_octave_notename(),
@@ -535,8 +531,6 @@ class Voice(VoiceCommon):
         return "\\staff{\n%s\n}" % self.m_musicdata
 
 class Rvoice(VoiceCommon):
-    def __init__(self, musicdata):
-        VoiceCommon.__init__(self, musicdata)
     def get_err_context(self, exception, lessonfile_ref):
         """
         Return a twoline string showing what caused the exception.
@@ -618,10 +612,7 @@ class Satb(ChordCommon):
         return music
     def play_arpeggio(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
-        instrument = lessonfile_ref.prepare_instrument_list(question)
-        track = soundcard.Track()
-        track.set_bpm(cfg.get_int('config/default_bpm'))
-        track.set_patch(instrument[0])
+        track = utils.new_track()
         voices = [n.strip() for n in self.m_musicdata.split('|')]
         for x in 0, 1:
             s = voices[x].strip().split(" ")
@@ -629,18 +620,18 @@ class Satb(ChordCommon):
                 if lessonfile_ref.header.random_transpose:
                     n = mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename()
                 if cfg.get_string('user/sex') == 'female':
-                    track.note(4, mpd.notename_to_int(n), instrument[1])
+                    track.note(4, mpd.notename_to_int(n))
                 else:
-                    track.note(4, mpd.notename_to_int(n)-12, instrument[1])
+                    track.note(4, mpd.notename_to_int(n)-12)
         for x in 2, 3:
             s = voices[x].strip().split(" ")
             for n in s:
                 if lessonfile_ref.header.random_transpose:
                     n = mpd.MusicalPitch.new_from_notename(n).transpose_by_musicalpitch(lessonfile_ref.m_transpose).get_octave_notename()
                 if cfg.get_string('user/sex') == 'male':
-                    track.note(4, mpd.notename_to_int(n), instrument[1])
+                    track.note(4, mpd.notename_to_int(n))
                 else:
-                    track.note(4, mpd.notename_to_int(n)+12, instrument[1])
+                    track.note(4, mpd.notename_to_int(n)+12)
         soundcard.synth.play_track(track)
     def get_err_context(self, exception, lessonfile_ref):
         """
@@ -673,15 +664,14 @@ class PercBaseClass(MpdParsable):
         return "\\staff{\n%s\n}" % self.m_musicdata
     def _gen_track(self, lessonfile_ref, question):
         score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
-        track = score.get_midi_events_as_percussion(cfg.get_int('config/preferred_instrument_velocity'))[0]
+        track = score.get_midi_events_as_percussion()[0]
+        track.set_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.prepend_bpm(lessonfile_ref.get_tempo()[0],
                           lessonfile_ref.get_tempo()[1])
         return track
 
 
 class Rhythm(PercBaseClass):
-    def __init__(self, musicdata):
-        PercBaseClass.__init__(self, musicdata)
     def play(self, lessonfile_ref, question):
         track = self._gen_track(lessonfile_ref, question)
         track.replace_note(mpd.notename_to_int("c"), 37)
@@ -690,8 +680,6 @@ class Rhythm(PercBaseClass):
 
 
 class Percussion(PercBaseClass):
-    def __init__(self, musicdata):
-        PercBaseClass.__init__(self, musicdata)
     def play(self, lessonfile_ref, question):
         soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
 
@@ -751,9 +739,14 @@ class Mma(MusicBaseClass):
             print >> f, "Transpose %i" % (lessonfile_ref.m_transpose.semitone_pitch() - mpd.MusicalPitch.new_from_notename("c'").semitone_pitch())
         f.write(self.m_musicdata)
         f.close()
-        mma = cfg.get_string("programs/mma")
-        if os.system("%(mma)s -f %(outfile)s %(infile)s" % locals()) != 0:
-            raise osutils.RunningExecutableFailed(mma)
+        try:
+            mma = [cfg.get_string("programs/mma"), "-f", outfile, infile]
+            if mma[0].endswith(".py"):
+                mma.insert(0, sys.executable)
+            subprocess.call(mma)
+        except OSError, e:
+            raise osutils.BinaryForProgramException("MMA",
+                cfg.get_string("programs/mma"), e)
         soundcard.play_mediafile('midi', os.path.join(self.temp_dir, 'f.mid'))
 
 class Wavfile(_MusicExternalPlayer):
@@ -769,8 +762,6 @@ class Oggfile(_MusicExternalPlayer):
         _MusicExternalPlayer.__init__(self, 'ogg', musicdata)
 
 class Cmdline(MusicBaseClass):
-    def __init__(self, musicdata):
-        MusicBaseClass.__init__(self, musicdata)
     def play(self, lessonfile_ref, question):
         assert isinstance(lessonfile_ref, LessonfileCommon)
         osutils.run_external_program(str(self.m_musicdata),
@@ -786,18 +777,19 @@ class CSound(MusicBaseClass):
         outfile = open(os.path.join(self.temp_dir, "in.sco"), 'w')
         outfile.write(self.m_musicdata[1])
         outfile.close()
-        if not os.path.exists(cfg.get_string("programs/csound")):
-            raise osutils.ExecutableDoesNotExist(cfg.get_string("programs/csound"))
-        os.system(cfg.get_string("programs/csound") + " %s %s -W -d -o %s" % (
-            os.path.join(self.temp_dir, "in.orc"),
-            os.path.join(self.temp_dir, "in.sco"),
-            os.path.join(self.temp_dir, "out.wav")))
+        try:
+            subprocess.call(
+                (cfg.get_string("programs/csound"),
+                 os.path.join(self.temp_dir, "in.orc"),
+                 os.path.join(self.temp_dir, "in.sco"),
+                 "-W", "-d", "-o",
+                 os.path.join(self.temp_dir, "out.wav")))
+        except OSError, e:
+             raise osutils.BinaryForProgramException("Csound", cfg.get_string("programs/csound"), e)
         soundcard.play_mediafile('wav', os.path.join(self.temp_dir, "out.wav"))
 
 
 class Music(MpdDisplayable):
-    def __init__(self, musicdata):
-        MpdDisplayable.__init__(self, musicdata)
     def get_mpd_music_string(self, lessonfile_ref):
         if lessonfile_ref.header.random_transpose:
             s = self.m_musicdata.replace(r'\staff',
@@ -806,30 +798,6 @@ class Music(MpdDisplayable):
                r'\addvoice\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
             return s
         return self.m_musicdata
-    def play(self, lessonfile_ref, question):
-        assert isinstance(lessonfile_ref, LessonfileCommon)
-        if len(lessonfile_ref.prepare_instrument_list(question)) == 2:
-            MpdParsable.play(self, lessonfile_ref, question)
-        else:
-            self.play_3patches(lessonfile_ref, question, 
-                    lessonfile_ref.get_tempo())
-    def play_slowly(self, lessonfile_ref, question):
-        assert isinstance(lessonfile_ref, LessonfileCommon)
-        if len(lessonfile_ref.prepare_instrument_list(question)) == 2:
-            MpdParsable.play_slowly(self, lessonfile_ref, question)
-        else:
-            tempo = lessonfile_ref.get_tempo()
-            tempo = (tempo[0]/2, tempo[1])
-            self.play_3patches(lessonfile_ref, question, tempo)
-    def play_3patches(self, lessonfile_ref, question, tempo):
-        """
-        Play the music with different instrument for the top and bottom voice.
-        Will use the instruments defined in the preferences window.
-        """
-        mpd.play_music3(
-            self.get_mpd_music_string(lessonfile_ref),
-            tempo,
-            lessonfile_ref.prepare_instrument_list(question))
     def get_err_context(self, exception, lessonfile_ref):
         """
         Return a twoline string showing what caused the exception.
@@ -849,6 +817,18 @@ class Music(MpdDisplayable):
         return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
             " " * first + "^" * (last - first)))
 
+
+class Music3(Music):
+    def play(self, lessonfile_ref, question):
+        """
+        Play the music with different instrument for the top and bottom voice.
+        Will use the instruments defined in the preferences window.
+        """
+        mpd.play_music3(
+            self.get_mpd_music_string(lessonfile_ref),
+            lessonfile_ref.get_tempo(),
+            lessonfile_ref.get_instruments(question, 3))
+
 def parse_test_def(s):
     m = re.match("(\d+)\s*x", s)
     count = int(m.groups()[0])
@@ -1134,51 +1114,57 @@ class QuestionsLessonfile(LessonfileCommon):
         question[varname].play_slowly(self, question)
     def play_question_arpeggio(self, varname='music'):
         self.get_question()[varname].play_arpeggio(self, self.get_question())
-    def prepare_instrument_list(self, question):
-        """Return a list created from the instrument variable the question.
-        Use app default values if the variable is missing.
-        Convert instrument names to integer values.
-        Returns: lowest, middle, highest
+    def get_instrument(self, question):
         """
-        if cfg.get_bool('config/override_default_instrument'):
-            instrument = [cfg.get_int('config/lowest_instrument'),
-                         cfg.get_int('config/lowest_instrument_velocity'),
-                         cfg.get_int('config/middle_instrument'),
-                         cfg.get_int('config/middle_instrument_velocity'),
-                         cfg.get_int('config/highest_instrument'),
-                         cfg.get_int('config/highest_instrument_velocity')]
-        elif 'instrument' in question:
-            instrument = question['instrument']
+        Music objects that will use the "config/preferred_instrument"
+        or the music instrument selected in the lesson file should
+        call this method.
+        """
+        if 'instrument' in question:
+            retval = question['instrument']
         elif 'instrument' in self.m_globals:
-            instrument = self.m_globals['instrument']
+            retval = self.m_globals['instrument']
         else:
-            instrument = [cfg.get_int('config/preferred_instrument'),
-                          cfg.get_int('config/preferred_instrument_velocity')]
-        if isinstance(instrument, (unicode, int)):
-            instrument = [instrument,
-                          cfg.get_int('config/preferred_instrument_velocity')]
-        assert len(instrument) in (2, 6)
-        if len(instrument) == 2:
-            if isinstance(instrument[0], unicode):
+            retval = [cfg.get_int('config/preferred_instrument'),
+                      cfg.get_int('config/preferred_instrument_volume')]
+        if isinstance(retval[0], unicode):
+            try:
+                retval[0] = soundcard.find_midi_instrument_number(retval[0])
+            except KeyError, e:
+                print >> sys.stderr, "Warning: Invalid instrument name '%s' in lesson file:" % retval[0], e
+                retval[0] = cfg.get_int('config/preferred_instrument')
+        return retval
+    def get_instruments(self, question, count):
+        """
+        Music objects that want 2 or 3 of the instruments we can configure
+        if we select "Override default instrument" should call this method.
+        """
+        assert count in (1, 2, 3)
+        if (count in (2, 3) and 
+            cfg.get_bool('config/override_default_instrument')):
+                retval = [cfg.get_int('config/lowest_instrument'),
+                          cfg.get_int('config/lowest_instrument_volume'),
+                          cfg.get_int('config/middle_instrument'),
+                          cfg.get_int('config/middle_instrument_volume'),
+                          cfg.get_int('config/highest_instrument'),
+                          cfg.get_int('config/highest_instrument_volume')]
+        else:
+            retval = [cfg.get_int('config/preferred_instrument'),
+                      cfg.get_int('config/preferred_instrument_volume')]
+        if 'instrument' in question:
+            retval = question['instrument']
+        elif 'instrument' in self.m_globals:
+            retval = self.m_globals['instrument']
+        while len(retval) < count * 2:
+            retval.extend(retval[-2:])
+        for idx in range(0, len(retval), 2):
+            if isinstance(retval[idx], unicode):
                 try:
-                    instrument[0] = soundcard.find_midi_instrument_number(instrument[0])
+                    retval[idx] = soundcard.find_midi_instrument_number(retval[idx])
                 except KeyError, e:
-                    print >> sys.stderr, "Warning: Invalid instrument name '%s' in lesson file:" % instrument[0], e
-                    instrument[0] = cfg.get_int('config/preferred_instrument')
-            if not (0 <= instrument[1] < 128):
-                print >> sys.stderr, "Warning: Adjusting instrument velocity since this value is invalid '%s'" % instrument[1]
-                instrument[1] = cfg.get_int('config/preferred_instrument_velocity')
-        elif len(instrument) == 6:
-            for x in (0, 2, 4):
-                if isinstance(instrument[x], unicode):
-                    try:
-                        instrument[x] = soundcard.find_midi_instrument_number(instrument[x])
-                    except KeyError, e:
-                        print "Warning: Invalid instrument name in lesson file:", e
-                        instrument[0] = cfg.get_int('config/preferred_instrument')
-                if not (0 <= instrument[x+1] < 128):
-                    instrument[x+1] = cfg.get_int('config/preferred_instrument_velocity')
-        return instrument
+                    print >> sys.stderr, "Warning: Invalid instrument name '%s' in lesson file:" % retval[idx], e
+                    retval[idx] = cfg.get_int('config/preferred_instrument')
+        return retval
     def discard_questions_without_name(self):
         # Delete questions that does not have a name
         q = self.m_questions
@@ -1320,9 +1306,6 @@ class NameIntervalLessonfile(HeaderLessonfile):
             self.header.tones = [mpd.MusicalPitch.new_from_notename(n) for n in self.header.tones]
 
 class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
-    def __init__(self):
-        QuestionsLessonfile.__init__(self)
-        TestSupport.__init__(self)
     def parse_string(self, s, really_filename=None):
         super(IdByNameLessonfile, self).parse_string(s, really_filename)
         # Also, if some questions has cuemusic, then we need the displayer
