@@ -22,7 +22,6 @@ import abstract
 import lessonfile
 import gu
 import mpd
-import utils
 
 class Teacher(abstract.Teacher):
     def __init__(self, exname, app):
@@ -119,7 +118,7 @@ class Teacher(abstract.Teacher):
 class Gui(abstract.LessonbasedGui):
     def __init__(self, teacher, window):
         abstract.LessonbasedGui.__init__(self, teacher, window, True)
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.g_music_displayer.show()
         self.practise_box.pack_start(self.g_music_displayer, False, False)
         self.g_flashbar = gu.FlashBar()
