@@ -16,10 +16,11 @@
 
 import gtk
 import gu
+import lessonfilegui
 
 
 class MatrixTable(gtk.VBox):
-    def __init__(self, label_style, heading, keys, key_to_pretty_name, dict):
+    def __init__(self, heading, keys, key_to_pretty_name, dict):
         gtk.VBox.__init__(self)
         label = gtk.Label(heading)
         label.set_name("StatisticsH2")
@@ -35,12 +36,8 @@ class MatrixTable(gtk.VBox):
         for x in range(len(keys)-1):
             t.attach(gtk.HSeparator(), 0, len(keys)*2+1, x*2+1, x*2+2)
         for y in range(len(keys)):
-            if label_style == 'progression':
-                l = gu.HarmonicProgressionLabel(
-                                     key_to_pretty_name(keys[y]), 'left')
-            else:
-                l = gtk.Label(key_to_pretty_name(keys[y]))
-                l.set_alignment(0.0, 0.5)
+            l = lessonfilegui.new_labelobject(key_to_pretty_name(keys[y]))
+            l.set_alignment(0.0, 0.5)
             t.attach(l, 0, 1, y*2, y*2+1, xpadding=gu.PAD)
             for x in range(len(keys)):
                 if keys[x] in dict and keys[y] in dict[keys[x]]:
@@ -95,11 +92,8 @@ class PercentagesTable(gtk.Frame):
                 print "WARNING: statistics: key==None. Please, please report to"
                 print "bug-solfege@gnu.org if you get this message. Preferably,"
                 print "make a copy ~/.solfege so I can debug this."
-            if statistics.get_label_style() == 'progression':
-                l = gu.HarmonicProgressionLabel(statistics.key_to_pretty_name(k), 'left')
-            else:
-                l = gtk.Label(statistics.key_to_pretty_name(k))
-                l.set_alignment(0.0, 0.5)
+            l = lessonfilegui.new_labelobject(statistics.key_to_pretty_name(k))
+            l.set_alignment(0.0, 0.5)
             self.boxdict['keys'].pack_start(l)
             for sk, st in (('session', statistics.m_session_stat),
                        ('today', statistics.m_today_stat),
@@ -174,25 +168,21 @@ class StatisticsViewer(NewAbstractStatisticsViewer):
         self.g_p.show_all()
         self.g_tables.pack_start(self.g_p, False)
         self.matrix_dict['session'] = MatrixTable(
-                                           self.m_statistics.get_label_style(), 
                                            _("Session"),
                                            self.m_statistics.get_keys(1),
                                            self.m_statistics.key_to_pretty_name,
                                            self.m_statistics.m_session_stat)
         self.matrix_dict['today'] = MatrixTable(
-                                           self.m_statistics.get_label_style(),
                                            _("Today"),
                                            self.m_statistics.get_keys(1),
                                            self.m_statistics.key_to_pretty_name,
                                            self.m_statistics.m_today_stat)
         self.matrix_dict['last7'] = MatrixTable(
-                                           self.m_statistics.get_label_style(),
                                            _("Last 7 days"),
                                            self.m_statistics.get_keys(1),
                                            self.m_statistics.key_to_pretty_name,
                                            self.m_statistics.m_last7_stat)
         self.matrix_dict['total'] = MatrixTable(
-                                           self.m_statistics.get_label_style(),
                                            _("Total"),
                                            self.m_statistics.get_keys(1),
                                            self.m_statistics.key_to_pretty_name,
