@@ -12,12 +12,11 @@ def do_file(filename):
     m = r.search(s)
     def func(r):
         years = r.group('years')
-        if '2007' not in years:
-            years ="%s, 2007" % years
+        if '2008' not in years:
+            years ="%s, 2008" % years
         return r.group('left') + years + r.group('right')
     s2 = r.sub(func, s)
-    print s2
-    if 0==1:#s2 != s:
+    if s2 != s:
         f = file(filename, 'w')
         f.write(s2)
         f.close()
